/**
 * Mixin for scope
 * @function scopeMixin
 */
'use strict'

const abind = require('abind')
const { ERROR_SCOPE_NAME } = require('../constants')

/** @lends scopeMixin */
function scopeMixin (BaseClass) {
  return class ScopeMixed extends BaseClass {
    constructor (...args) {
      super(...args)
      const s = this
      abind(s, { proto: ScopeMixed.prototype })
    }

    /**
     * Value for scope
     * @param {..string} keys - Key path
     * @returns {*}
     */
    scopeValue (...keys) {
      const s = this
      s.assertScopeName()
      let { scopeName } = s
      return s.get(scopeName, ...keys)
    }

    /**
     * Error for scope
     * @param {..string} keys - Key path
     * @returns {*}
     */
    scopeError (...keys) {
      const s = this
      s.assertScopeName()
      let { scopeName } = s
      return s.get(ERROR_SCOPE_NAME, scopeName, ...keys)
    }

    /**
     * Dispatch operation on scope
     * @param {string} operationType - Type of the operation
     * @param {string} fieldName - Name of field
     * @param {Object} [value] - Value to dispatch
     * @param {boolean} [isError=false] - Error or not
     */
    scopeOperate (operationType, fieldName, value, isError = false) {
      const s = this
      s.assertScopeName()
      let { scopeName } = s
      let actionType = `${scopeName}:${operationType}`.trim()
      s.dispatch(actionType, fieldName, value, isError)
    }

    assertScopeName () {
      const s = this
      let { scopeName } = s
      if (!scopeName) {
        throw new Error('[ScopeMixin] scopeName is missing!')
      }
    }
  }
}

module.exports = scopeMixin
